<?php

namespace FiloBlu\Refilo\Remote;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class AbstractWebsiteBasedIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractWebsiteBasedIndexer extends AbstractIndexer
{

    /**
     * @param array $ids
     * @return mixed|void
     * @throws NoSuchEntityException
     */
    public function executeAction(array $ids)
    {
        try {
            $indexTemplate = $this->getGetRemoteIndexTemplate();
        } catch (\Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
            return;
        }

        foreach ($this->getStoreManager()->getWebsites() as $website) {
            $entities = $this->entityProviderFactory->create()
                ->forWebsite($website)
                ->withIds($ids);

            $previousStore = $this->getStoreManager()->getStore();
            /** @var StoreInterface $store */
            foreach ($website->getStores() as $store) {
                $this->getStoreManager()->setCurrentStore($store);
                if (!$store->getIsActive()) {
                    continue;
                }
                $collection = $indexTemplate->withValue('%s', $store->getCode())->render();
                $this->write($entities, $collection);
            }
            $this->getStoreManager()->setCurrentStore($previousStore);
        }
    }
}
